@@ -4,7 +4,6 @@
 Simple gLite job manager. See https://wiki.italiangrid.it/twiki/bin/view/CREAM/UserGuide.
 """
 
-
 __all__ = ["GLiteJobManager", "GLiteJobFileFactory"]
 
 
@@ -14,15 +13,15 @@ import time
 import re
 import random
 import subprocess
-import logging
 
 from law.config import Config
 from law.job.base import BaseJobManager, BaseJobFileFactory
 from law.target.file import add_scheme
 from law.util import interruptable_popen, make_list, make_unique, quote_cmd
+from law.logger import get_logger
 
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 _cfg = Config.instance()
 
