@@ -5,13 +5,11 @@ gLite remote workflow implementation. See
 https://wiki.italiangrid.it/twiki/bin/view/CREAM/UserGuide.
 """
 
-
 __all__ = ["GLiteWorkflow"]
 
 
 import os
 import sys
-import logging
 from abc import abstractmethod
 from collections import OrderedDict
 
@@ -22,11 +20,12 @@ from law.target.file import get_path
 from law.parameter import CSVParameter
 from law.util import law_src_path, merge_dicts, DotDict
 from law.contrib.wlcg import delegate_voms_proxy_glite, get_ce_endpoint
+from law.logger import get_logger
 
 from law.contrib.glite.job import GLiteJobManager, GLiteJobFileFactory
 
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 
 class GLiteWorkflowProxy(BaseRemoteWorkflowProxy):
@@ -207,14 +206,20 @@ class GLiteWorkflow(BaseRemoteWorkflow):
         return delegate_voms_proxy_glite(endpoint, stdout=sys.stdout, stderr=sys.stderr,
             cache=True)
 
+    def glite_job_manager_cls(self):
+        return GLiteJobManager
+
     def glite_create_job_manager(self, **kwargs):
         kwargs = merge_dicts(self.glite_job_manager_defaults, kwargs)
-        return GLiteJobManager(**kwargs)
+        return self.glite_job_manager_cls()(**kwargs)
+
+    def glite_job_file_factory_cls(self):
+        return GLiteJobFileFactory
 
     def glite_create_job_file_factory(self, **kwargs):
         # job file fectory config priority: kwargs > class defaults
         kwargs = merge_dicts({}, self.glite_job_file_factory_defaults, kwargs)
-        return GLiteJobFileFactory(**kwargs)
+        return self.glite_job_file_factory_cls()(**kwargs)
 
     def glite_job_config(self, config, job_num, branches):
         return config
