@@ -4,7 +4,6 @@
 HTCondor job manager. See https://research.cs.wisc.edu/htcondor.
 """
 
-
 __all__ = ["HTCondorJobManager", "HTCondorJobFileFactory"]
 
 
@@ -13,14 +12,14 @@ import stat
 import time
 import re
 import subprocess
-import logging
 
 from law.config import Config
 from law.job.base import BaseJobManager, BaseJobFileFactory
 from law.util import interruptable_popen, make_list, make_unique, quote_cmd
+from law.logger import get_logger
 
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 _cfg = Config.instance()
 
@@ -53,7 +52,7 @@ class HTCondorJobManager(BaseJobManager):
     @classmethod
     def get_htcondor_version(cls):
         code, out, _ = interruptable_popen("condor_version", shell=True, executable="/bin/bash",
-            stdout=subprocess.PIPE)
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         if code == 0:
             m = re.match(r"^\$CondorVersion: (\d+)\.(\d+)\.(\d+) .+$", out.split("\n")[0].strip())
             if m:
@@ -220,7 +219,7 @@ class HTCondorJobManager(BaseJobManager):
                         "\n{}".format(job_id, code, err))
 
             # parse the output and update query data
-            query_data.update(self.parse_long_output(out, job_ids=missing_ids))
+            query_data.update(self.parse_long_output(out))
 
         # compare to the requested job ids and perform some checks
         for _job_id in job_ids:
@@ -238,7 +237,7 @@ class HTCondorJobManager(BaseJobManager):
         return query_data if chunking else query_data[job_id]
 
     @classmethod
-    def parse_long_output(cls, out, job_ids=None):
+    def parse_long_output(cls, out):
         # retrieve information per block mapped to the job id
         query_data = {}
         for block in out.strip().split("\n\n"):
