@@ -4,7 +4,6 @@
 LSF job manager. See https://www.ibm.com/support/knowledgecenter/en/SSETD4_9.1.3.
 """
 
-
 __all__ = ["LSFJobManager", "LSFJobFileFactory"]
 
 
@@ -12,16 +11,16 @@ import os
 import time
 import re
 import subprocess
-import logging
 
 import six
 
 from law.config import Config
 from law.job.base import BaseJobManager, BaseJobFileFactory
 from law.util import interruptable_popen, make_list, make_unique, quote_cmd
+from law.logger import get_logger
 
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 _cfg = Config.instance()
 
