@@ -4,12 +4,10 @@
 LSF remote workflow implementation. See https://www.ibm.com/support/knowledgecenter/en/SSETD4_9.1.3.
 """
 
-
 __all__ = ["LSFWorkflow"]
 
 
 import os
-import logging
 from abc import abstractmethod
 from collections import OrderedDict
 
@@ -22,11 +20,12 @@ from law.target.file import get_path
 from law.target.local import LocalDirectoryTarget
 from law.parameter import NO_STR
 from law.util import law_src_path, merge_dicts, DotDict
+from law.logger import get_logger
 
 from law.contrib.lsf.job import LSFJobManager, LSFJobFileFactory
 
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 
 class LSFWorkflowProxy(BaseRemoteWorkflowProxy):
@@ -186,14 +185,20 @@ class LSFWorkflow(BaseRemoteWorkflow):
     def lsf_output_postfix(self):
         return "_" + self.get_branches_repr()
 
+    def lsf_job_manager_cls(self):
+        return LSFJobManager
+
     def lsf_create_job_manager(self, **kwargs):
         kwargs = merge_dicts(self.lsf_job_manager_defaults, kwargs)
-        return LSFJobManager(**kwargs)
+        return self.lsf_job_manager_cls()(**kwargs)
+
+    def lsf_job_file_factory_cls(self):
+        return LSFJobFileFactory
 
     def lsf_create_job_file_factory(self, **kwargs):
         # job file fectory config priority: kwargs > class defaults
         kwargs = merge_dicts({}, self.lsf_job_file_factory_defaults, kwargs)
-        return LSFJobFileFactory(**kwargs)
+        return self.lsf_job_file_factory_cls()(**kwargs)
 
     def lsf_job_config(self, config, job_num, branches):
         return config
