@@ -4,17 +4,15 @@
 NumPy target formatters.
 """
 
-
 __all__ = ["NumpyFormatter"]
 
 
-import logging
-
 from law.target.formatter import Formatter
 from law.target.file import get_path
+from law.logger import get_logger
 
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 
 class NumpyFormatter(Formatter):
