@@ -4,17 +4,15 @@
 Notification functions.
 """
 
-
 __all__ = ["notify_mail"]
 
 
-import logging
-
 from law.config import Config
-from law.util import send_mail
+from law.util import send_mail, uncolored
+from law.logger import get_logger
 
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 
 def notify_mail(title, message, recipient=None, sender=None, smtp_host=None, smtp_port=None,
@@ -45,4 +43,4 @@ def notify_mail(title, message, recipient=None, sender=None, smtp_host=None, smt
     if smtp_port:
         mail_kwargs["smtp_port"] = smtp_port
 
-    return send_mail(recipient, sender, title, message, **mail_kwargs)
+    return send_mail(recipient, sender, title, uncolored(message), **mail_kwargs)
