@@ -4,13 +4,11 @@
 Abstract defintions that enable task sandboxing.
 """
 
-
 __all__ = ["Sandbox", "SandboxTask"]
 
 
 import os
 import sys
-import logging
 from abc import ABCMeta, abstractmethod, abstractproperty
 from contextlib import contextmanager
 from fnmatch import fnmatch
@@ -30,9 +28,10 @@ from law.util import (
     colored, is_pattern, multi_match, mask_struct, map_struct, interruptable_popen, patch_object,
     flatten,
 )
+from law.logger import get_logger
 
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 _current_sandbox = os.getenv("LAW_SANDBOX", "").split(",")
 
@@ -81,8 +80,7 @@ class StageInfo(object):
         return str(self)
 
 
-@six.add_metaclass(ABCMeta)
-class Sandbox(object):
+class Sandbox(six.with_metaclass(ABCMeta, object)):
 
     delimiter = "::"
 
