@@ -4,26 +4,24 @@
 Custom luigi file system and target objects.
 """
 
-
 __all__ = [
     "FileSystem", "FileSystemTarget", "FileSystemFileTarget", "FileSystemDirectoryTarget",
-    "get_path", "get_scheme", "has_scheme", "add_scheme", "remove_scheme", "split_transfer_kwargs",
-    "localize_file_targets",
+    "get_path", "get_scheme", "has_scheme", "add_scheme", "remove_scheme", "localize_file_targets",
 ]
 
 
 import os
 import sys
+import re
 from abc import abstractmethod, abstractproperty
 from contextlib import contextmanager
 
-import six
 import luigi
 import luigi.task
 
 from law.config import Config
 from law.target.base import Target
-from law.util import create_hash, make_list, map_struct
+from law.util import map_struct, create_random_string
 
 
 class FileSystem(luigi.target.FileSystem):
@@ -41,19 +39,19 @@ class FileSystem(luigi.target.FileSystem):
             if option not in config or overwrite:
                 config[option] = func(section, option)
 
-        # permissions
-        add("has_perms", cfg.get_expanded_boolean)
+        # read configs
+        add("has_permissions", cfg.get_expanded_boolean)
         add("default_file_perm", cfg.get_expanded_int)
         add("default_dir_perm", cfg.get_expanded_int)
         add("create_file_dir", cfg.get_expanded_boolean)
 
         return config
 
-    def __init__(self, has_perms=True, default_file_perm=None, default_dir_perm=None,
+    def __init__(self, has_permissions=True, default_file_perm=None, default_dir_perm=None,
             create_file_dir=True, **kwargs):
         luigi.target.FileSystem.__init__(self)
 
-        self.has_perms = has_perms
+        self.has_permissions = has_permissions
         self.default_file_perm = default_file_perm
         self.default_dir_perm = default_dir_perm
         self.create_file_dir = create_file_dir
@@ -61,18 +59,12 @@ class FileSystem(luigi.target.FileSystem):
     def __repr__(self):
         return "{}({})".format(self.__class__.__name__, hex(id(self)))
 
-    def hash(self, path, l=10):
-        return create_hash(self.__class__.__name__ + self.abspath(path))
-
     def dirname(self, path):
-        return os.path.dirname(self.abspath(path)) if path != "/" else None
+        return os.path.dirname(path) if path != "/" else None
 
     def basename(self, path):
         return os.path.basename(path) if path != "/" else "/"
 
-    def unique_basename(self, path, l=10):
-        return self.hash(path, l=l) + "_" + self.basename(path)
-
     def ext(self, path, n=1):
         # split the path
         parts = self.basename(path).lstrip(".").split(".")
@@ -83,10 +75,6 @@ class FileSystem(luigi.target.FileSystem):
         else:
             return ".".join(parts[1:][min(-n, 0):])
 
-    @abstractmethod
-    def __eq__(self, other):
-        return
-
     @abstractproperty
     def default_instance(self):
         return
@@ -96,11 +84,11 @@ class FileSystem(luigi.target.FileSystem):
         return
 
     @abstractmethod
-    def stat(self, path):
+    def stat(self, path, **kwargs):
         return
 
     @abstractmethod
-    def exists(self, path):
+    def exists(self, path, **kwargs):
         return
 
     @abstractmethod
@@ -145,7 +133,7 @@ class FileSystem(luigi.target.FileSystem):
 
     @abstractmethod
     @contextmanager
-    def open(self, path, mode, **kwargs):
+    def open(self, path, mode, perm=None, dir_perm=None, **kwargs):
         return
 
     @abstractmethod
@@ -166,43 +154,31 @@ class FileSystemTarget(Target, luigi.target.FileSystemTarget):
         if fs:
             self.fs = fs
 
+        self._path = None
+        self._unexpanded_path = None
+
         Target.__init__(self, **kwargs)
         luigi.target.FileSystemTarget.__init__(self, path)
 
-    def __eq__(self, other):
-        return self.__class__ == other.__class__ and \
-            self.fs == other.fs and \
-            self.fs.abspath(self.path) == other.fs.abspath(other.path)
-
     def _repr_pairs(self, color=True):
-        return Target._repr_pairs(self) + [("path", self.path)]
+        expand = Config.instance().get_expanded_boolean("target", "expand_path_repr")
+        return Target._repr_pairs(self) + [("path", self.path if expand else self.unexpanded_path)]
 
     def _parent_args(self):
         return (), {}
 
     @property
-    def hash(self):
-        return self.fs.hash(self.path)
-
-    def exists(self):
-        return self.fs.exists(self.path)
+    def unexpanded_path(self):
+        return self._unexpanded_path
 
     @property
-    def parent(self):
-        dirname = self.dirname
-        args, kwargs = self._parent_args()
-        return self.directory_class(dirname, *args, **kwargs) if dirname is not None else None
+    def path(self):
+        return self._path
 
-    def sibling(self, *args, **kwargs):
-        parent = self.parent
-        if not parent:
-            raise Exception("cannot determine file parent")
-
-        return parent.child(*args, **kwargs)
-
-    @property
-    def stat(self):
-        return self.fs.stat(self.path)
+    @path.setter
+    def path(self, path):
+        self._unexpanded_path = str(path)
+        self._path = os.path.abspath(os.path.expandvars(os.path.expanduser(self.unexpanded_path)))
 
     @property
     def dirname(self):
@@ -214,7 +190,32 @@ class FileSystemTarget(Target, luigi.target.FileSystemTarget):
 
     @property
     def unique_basename(self):
-        return self.fs.unique_basename(self.path)
+        return "{}_{}".format(hex(self.hash)[2:], self.basename)
+
+    @property
+    def parent(self):
+        # get the dirname, but favor the unexpanded one to propagate variables
+        dirname = self.dirname
+        unexpanded_dirname = self.fs.dirname(self.unexpanded_path)
+        expanded_dirname = os.path.expandvars(os.path.expanduser(unexpanded_dirname))
+        if unexpanded_dirname and dirname == os.path.abspath(expanded_dirname):
+            dirname = unexpanded_dirname
+
+        args, kwargs = self._parent_args()
+        return self.directory_class(dirname, *args, **kwargs) if dirname is not None else None
+
+    def sibling(self, *args, **kwargs):
+        parent = self.parent
+        if not parent:
+            raise Exception("cannot determine parent of {!r}".format(self))
+
+        return parent.child(*args, **kwargs)
+
+    def exists(self, **kwargs):
+        return self.fs.exists(self.path, **kwargs)
+
+    def stat(self, **kwargs):
+        return self.fs.stat(self.path, **kwargs)
 
     def remove(self, silent=True, **kwargs):
         self.fs.remove(self.path, silent=silent, **kwargs)
@@ -227,7 +228,7 @@ class FileSystemTarget(Target, luigi.target.FileSystemTarget):
         return
 
     @abstractmethod
-    def touch(self):
+    def touch(self, perm=None, dir_perm=None):
         return
 
 
@@ -238,19 +239,10 @@ class FileSystemFileTarget(FileSystemTarget):
     def ext(self, n=1):
         return self.fs.ext(self.path, n=n)
 
-    def touch(self, content="", perm=None, dir_perm=None, **kwargs):
-        # create the parent
-        parent = self.parent
-        if parent is not None:
-            parent.touch(perm=dir_perm, **kwargs)
-
-        # create the file via open and write content
+    def touch(self, **kwargs):
+        # create the file via open without content
         with self.open("w", **kwargs) as f:
-            f.write(content)
-
-        if perm is None:
-            perm = self.fs.default_file_perm
-        self.chmod(perm, **kwargs)
+            f.write("")
 
     def open(self, mode, **kwargs):
         return self.fs.open(self.path, mode, **kwargs)
@@ -303,15 +295,21 @@ class FileSystemDirectoryTarget(FileSystemTarget):
 
     open = None
 
-    def _child_args(self):
+    def _child_args(self, path):
         return (), {}
 
-    def child(self, path, type=None, **kwargs):
+    def child(self, path, type=None, mktemp_pattern=False, **kwargs):
         if type not in (None, "f", "d"):
             raise ValueError("invalid child type, use 'f' or 'd'")
 
-        path = os.path.join(self.path, get_path(path))
+        # apply mktemp's feature to replace at least three consecutive 'X' with random characters
+        path = get_path(path)
+        if mktemp_pattern and "XXX" in path:
+            repl = lambda m: create_random_string(l=len(m.group(1)))
+            path = re.sub("(X{3,})", repl, path)
 
+        unexpanded_path = os.path.join(self.unexpanded_path, path)
+        path = os.path.join(self.path, path)
         if type == "f":
             cls = self.file_class
         elif type == "d":
@@ -323,12 +321,13 @@ class FileSystemDirectoryTarget(FileSystemTarget):
         else:
             cls = self.file_class
 
-        args, _kwargs = self._child_args()
+        args, _kwargs = self._child_args(path)
         _kwargs.update(kwargs)
-        return cls(path, *args, **_kwargs)
 
-    def listdir(self, pattern=None, type=None, **kwargs):
-        return self.fs.listdir(self.path, pattern=pattern, type=type, **kwargs)
+        return cls(unexpanded_path, *args, **_kwargs)
+
+    def listdir(self, **kwargs):
+        return self.fs.listdir(self.path, **kwargs)
 
     def glob(self, pattern, **kwargs):
         return self.fs.glob(pattern, cwd=self.path, **kwargs)
@@ -336,8 +335,9 @@ class FileSystemDirectoryTarget(FileSystemTarget):
     def walk(self, **kwargs):
         return self.fs.walk(self.path, **kwargs)
 
-    def touch(self, perm=None, recursive=True, **kwargs):
-        self.fs.mkdir(self.path, perm=perm, recursive=recursive, silent=True, **kwargs)
+    def touch(self, **kwargs):
+        kwargs.setdefault("silent", True)
+        self.fs.mkdir(self.path, **kwargs)
 
 
 FileSystemTarget.file_class = FileSystemFileTarget
@@ -351,7 +351,8 @@ def get_path(target):
 def get_scheme(uri):
     # ftp://path/to/file -> ftp
     # /path/to/file -> None
-    return six.moves.urllib_parse.urlparse(uri).scheme or None
+    m = re.match(r"^(\w+)\:\/\/.*$", uri)
+    return m.group(1) if m else None
 
 
 def has_scheme(uri):
@@ -366,24 +367,7 @@ def add_scheme(path, scheme):
 def remove_scheme(uri):
     # ftp://path/to/file -> /path/to/file
     # /path/to/file -> /path/to/file
-    return six.moves.urllib_parse.urlparse(uri).path or None
-
-
-def split_transfer_kwargs(kwargs, skip=None):
-    """
-    Takes keyword arguments *kwargs*, splits them into two separate dictionaries depending on their
-    content, and returns them in a tuple. The first one will contain arguments related to potential
-    file transfer operations (e.g. ``"cache"`` or ``"retries"``), while the second one will contain
-    all remaining arguments. This function is used internally to decide which arguments to pass to
-    target formatters. *skip* can be a list of argument keys that are ignored.
-    """
-    skip = make_list(skip) if skip else []
-    transfer_kwargs = {
-        name: kwargs.pop(name)
-        for name in ["cache", "prefer_cache", "retries", "retry_delay"]
-        if name in kwargs and name not in skip
-    }
-    return transfer_kwargs, kwargs
+    return re.sub(r"^(\w+\:\/\/)", "", uri)
 
 
 @contextmanager
