@@ -4,7 +4,6 @@
 Local target implementations.
 """
 
-
 __all__ = ["LocalFileSystem", "LocalTarget", "LocalFileTarget", "LocalDirectoryTarget"]
 
 
@@ -13,7 +12,6 @@ import fnmatch
 import shutil
 import glob
 import random
-import logging
 from contextlib import contextmanager
 
 import luigi
@@ -22,13 +20,14 @@ import six
 from law.config import Config
 from law.target.file import (
     FileSystem, FileSystemTarget, FileSystemFileTarget, FileSystemDirectoryTarget, get_path,
-    get_scheme, add_scheme, remove_scheme, split_transfer_kwargs,
+    get_scheme, add_scheme, remove_scheme,
 )
 from law.target.formatter import find_formatter
 from law.util import is_file_exists_error
+from law.logger import get_logger
 
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 
 class LocalFileSystem(FileSystem):
@@ -55,9 +54,6 @@ class LocalFileSystem(FileSystem):
 
         FileSystem.__init__(self, **kwargs)
 
-    def __eq__(self, other):
-        return self.__class__ == other.__class__
-
     def _unscheme(self, path):
         return remove_scheme(path) if get_scheme(path) == "file" else path
 
@@ -77,46 +73,61 @@ class LocalFileSystem(FileSystem):
         return os.path.isfile(self._unscheme(path))
 
     def chmod(self, path, perm, silent=True, **kwargs):
-        if self.has_perms and perm is not None and (not silent or self.exists(path)):
-            os.chmod(self._unscheme(path), perm)
+        if not self.has_permissions or perm is None:
+            return True
+
+        if silent and not self.exists(path):
+            return False
+
+        os.chmod(self._unscheme(path), perm)
+
+        return True
 
     def remove(self, path, recursive=True, silent=True, **kwargs):
-        if not silent or self.exists(path):
-            path = self._unscheme(path)
-            if self.isdir(path):
-                if recursive:
-                    shutil.rmtree(path)
-                else:
-                    os.rmdir(path)
+        path = self._unscheme(path)
+
+        if silent and not self.exists(path):
+            return False
+
+        if self.isdir(path):
+            if recursive:
+                shutil.rmtree(path)
             else:
-                os.remove(path)
+                os.rmdir(path)
+        else:
+            os.remove(path)
+
+        return True
 
     def mkdir(self, path, perm=None, recursive=True, silent=True, **kwargs):
-        if self.exists(path):
-            return
+        if silent and self.exists(path):
+            return False
 
         if perm is None:
             perm = self.default_dir_perm
 
+        # prepare arguments passed to makedirs or mkdir
+        args = (self._unscheme(path),)
+        if perm is not None:
+            args += (perm,)
+
         # the mode passed to os.mkdir or os.makedirs is ignored on some systems, so the strategy
         # here is to disable the process' current umask, create the directories and use chmod again
-        if perm is not None:
-            orig = os.umask(0)
-
+        orig = os.umask(0) if perm is not None else None
+        func = os.makedirs if recursive else os.mkdir
         try:
-            args = (self._unscheme(path),)
-            if perm is not None:
-                args += (perm,)
             try:
-                (os.makedirs if recursive else os.mkdir)(*args)
+                func(*args)
             except Exception as e:
-                if not silent and not is_file_exists_error(e):
+                if not silent or not is_file_exists_error(e):
                     raise
             self.chmod(path, perm)
         finally:
-            if perm is not None:
+            if orig is not None:
                 os.umask(orig)
 
+        return True
+
     def listdir(self, path, pattern=None, type=None, **kwargs):
         path = self._unscheme(path)
         elems = os.listdir(path)
@@ -173,16 +184,16 @@ class LocalFileSystem(FileSystem):
 
         return elems
 
-    def _prepare_dst_dir(self, dst, src=None, perm=None):
+    def _prepare_dst_dir(self, dst, src=None, perm=None, **kwargs):
         """
-        Prepares the directory of a target located at *dst* and returns its full location as
-        specified below. *src* can be the location of a source file target, which is (e.g.) used by
-        a file copy or move operation. When *dst* is already a directory, calling this method has no
-        effect and the *dst* path is returned, optionally joined with the basename of *src*. When
-        *dst* is a file, the absolute *dst* path is returned. Otherwise, when *dst* does not exist
-        yet, it is interpreted as a file path and missing directories are created when
-        :py:attr:`create_file_dir` is *True*, using *perm* to set the directory permission. *dst* is
-        returned.
+        Prepares the directory of a target located at *dst* for copying and returns its full
+        location as specified below. *src* can be the location of a source file target, which is
+        (e.g.) used by a file copy or move operation. When *dst* is already a directory, calling
+        this method has no effect and the *dst* path is returned, optionally joined with the
+        basename of *src*. When *dst* is a file, the absolute *dst* path is returned. Otherwise,
+        when *dst* does not exist yet, it is interpreted as a file path and missing directories are
+        created when :py:attr:`create_file_dir` is *True*, using *perm* to set the directory
+        permission. The absolute path to *dst* is returned.
         """
         dst = self._unscheme(dst)
 
@@ -198,7 +209,7 @@ class LocalFileSystem(FileSystem):
         else:
             # interpret dst as a file name, create missing dirs
             dst_dir = self.dirname(dst)
-            if dst_dir and not self.isdir(dst_dir) and self.create_file_dir:
+            if dst_dir and self.create_file_dir and not self.isdir(dst_dir):
                 self.mkdir(dst_dir, perm=perm, recursive=True)
             full_dst = dst
 
@@ -232,18 +243,45 @@ class LocalFileSystem(FileSystem):
 
         return dst
 
-    def open(self, path, mode, **kwargs):
-        return open(self._prepare_dst_dir(path), mode)
+    def open(self, path, mode, perm=None, dir_perm=None, **kwargs):
+        # check if the file is only read
+        read_mode = mode.startswith("r")
+
+        if read_mode:
+            return open(path, mode)
+
+        else:  # write or update
+            # prepare the destination directory
+            self._prepare_dst_dir(path, perm=dir_perm)
+
+            if perm is None:
+                perm = self.default_file_perm
+
+            # when setting permissions, ensure the file exists first
+            if perm is not None and self.has_permissions:
+                open(path, mode).close()
+                self.chmod(path, perm)
+
+            return open(path, mode)
 
     def load(self, path, formatter, *args, **kwargs):
-        _, kwargs = split_transfer_kwargs(kwargs)
-        path = self._unscheme(path)
-        return find_formatter(path, "load", formatter).load(path, *args, **kwargs)
+        # remove kwargs that might be designated for remote files
+        kwargs = RemoteFileSystem.split_remote_kwargs(kwargs)[1]
+
+        return find_formatter(path, "load", formatter).load(self._unscheme(path), *args, **kwargs)
 
     def dump(self, path, formatter, *args, **kwargs):
-        _, kwargs = split_transfer_kwargs(kwargs)
-        path = self._prepare_dst_dir(path)
-        return find_formatter(path, "dump", formatter).dump(path, *args, **kwargs)
+        # remove kwargs that might be designated for remote files
+        kwargs = RemoteFileSystem.split_remote_kwargs(kwargs)[1]
+
+        # also remove permission settings
+        perm = kwargs.pop("perm", None)
+        dir_perm = kwargs.pop("dir_perm", None)
+
+        # use open() to handle parent directory writing and permissisions
+        self.open(path, "w", perm=perm, dir_perm=dir_perm).close()
+
+        return find_formatter(path, "dump", formatter).dump(self._unscheme(path), *args, **kwargs)
 
 
 LocalFileSystem.default_instance = LocalFileSystem()
@@ -286,12 +324,14 @@ class LocalTarget(FileSystemTarget, luigi.LocalTarget):
                     is_tmp = "." + is_tmp
                 path += is_tmp
         else:
-            # ensure path is not a target and does not contain, then normalize
+            # ensure path is not a target and does not contain a scheme
             path = fs._unscheme(get_path(path))
-            path = fs.abspath(os.path.expandvars(os.path.expanduser(path)))
+            # make absolute when not starting with a variable
+            if not path.startswith("$"):
+                path = os.path.abspath(path)
 
         luigi.LocalTarget.__init__(self, path=path, is_tmp=is_tmp)
-        FileSystemTarget.__init__(self, self.path, fs=fs, **kwargs)
+        FileSystemTarget.__init__(self, self.unexpanded_path, fs=fs, **kwargs)
 
     def _repr_flags(self):
         flags = FileSystemTarget._repr_flags(self)
@@ -299,8 +339,11 @@ class LocalTarget(FileSystemTarget, luigi.LocalTarget):
             flags.append("temporary")
         return flags
 
-    def uri(self, *args, **kwargs):
-        return add_scheme(self.fs.abspath(self.path), "file")
+    def uri(self, scheme=True, return_all=False, **kwargs):
+        uri = self.fs.abspath(self.path)
+        if scheme:
+            uri = add_scheme(uri, "file")
+        return [uri] if return_all else uri
 
 
 class LocalFileTarget(LocalTarget, FileSystemFileTarget):
@@ -319,9 +362,7 @@ class LocalFileTarget(LocalTarget, FileSystemFileTarget):
 
     @contextmanager
     def localize(self, mode="r", perm=None, dir_perm=None, tmp_dir=None, **kwargs):
-        """ localize(mode="r", perm=None, dir_perm=None, tmp_dir=None, is_tmp=None, **kwargs)
-        """
-        if mode not in ("r", "w", "a"):
+        if mode not in ["r", "w", "a"]:
             raise Exception("unknown mode '{}', use 'r', 'w' or 'a'".format(mode))
 
         logger.debug("localizing file target {!r} with mode '{}'".format(self, mode))
@@ -341,7 +382,7 @@ class LocalFileTarget(LocalTarget, FileSystemFileTarget):
                 try:
                     yield tmp
                 finally:
-                    tmp.remove()
+                    tmp.remove(silent=True)
             else:
                 # simply yield
                 yield self
@@ -374,8 +415,7 @@ class LocalFileTarget(LocalTarget, FileSystemFileTarget):
                 # simply yield
                 yield self
 
-                if self.exists():
-                    self.chmod(perm)
+                self.chmod(perm, silent=True)
 
 
 class LocalDirectoryTarget(LocalTarget, FileSystemDirectoryTarget):
@@ -385,3 +425,7 @@ class LocalDirectoryTarget(LocalTarget, FileSystemDirectoryTarget):
 
 LocalTarget.file_class = LocalFileTarget
 LocalTarget.directory_class = LocalDirectoryTarget
+
+
+# trailing imports
+from law.target.remote.base import RemoteFileSystem
