@@ -4,7 +4,6 @@
 Custom luigi base task definitions.
 """
 
-
 __all__ = ["Task", "WrapperTask", "ExternalTask"]
 
 
@@ -26,12 +25,13 @@ from law.target.file import localize_file_targets
 from law.parser import root_task, global_cmdline_values
 from law.logger import setup_logger
 from law.util import (
-    no_value, abort, common_task_params, colored, uncolored, make_list, multi_match, flatten,
-    BaseStream, human_duration, patch_object, round_discrete, classproperty,
+    no_value, abort, law_run, common_task_params, colored, uncolored, make_list, multi_match,
+    flatten, BaseStream, human_duration, patch_object, round_discrete,
 )
+from law.logger import get_logger
 
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 
 class BaseRegister(luigi.task_register.Register):
@@ -74,8 +74,7 @@ class BaseRegister(luigi.task_register.Register):
         return cls
 
 
-@six.add_metaclass(BaseRegister)
-class BaseTask(luigi.Task):
+class BaseTask(six.with_metaclass(BaseRegister, luigi.Task)):
 
     exclude_index = True
     exclude_params_index = set()
@@ -98,11 +97,11 @@ class BaseTask(luigi.Task):
         """
         # always compare task families
         if task_cls is None:
-            task_family = cls.task_family
+            task_family = cls.get_task_family()
         elif isinstance(task_cls, six.string_types):
             task_family = task_cls
         else:
-            task_family = task_cls.task_family
+            task_family = task_cls.get_task_family()
 
         success = False
 
@@ -114,7 +113,7 @@ class BaseTask(luigi.Task):
                 break
             registered_cls = Register._reg[i]
 
-            if multi_match(registered_cls.task_family, task_family, mode=any):
+            if multi_match(registered_cls.get_task_family(), task_family, mode=any):
                 Register._reg.pop(i)
                 i -= 1
                 success = True
@@ -123,18 +122,34 @@ class BaseTask(luigi.Task):
         return success
 
     @classmethod
-    def get_param_values(cls, *args, **kwargs):
-        values = super(BaseTask, cls).get_param_values(*args, **kwargs)
+    def get_param_values(cls, params, args, kwargs):
+        # try to modify the values before values are assigned
+        if callable(cls.modify_param_args):
+            params, args, kwargs = cls.modify_param_args(params, args, kwargs)
+
+        # assign to actual parameters
+        values = super(BaseTask, cls).get_param_values(params, args, kwargs)
+
+        # try to modify the values afterwards
         if callable(cls.modify_param_values):
-            return list(cls.modify_param_values(OrderedDict(values)).items())
-        else:
-            return values
+            values = list(cls.modify_param_values(OrderedDict(values)).items())
+
+        return values
+
+    # method that can be implemented to update parameter objects, args and kwargs before before task
+    # instantiation and before parameter values are assined in the super get_param_values
+    # example:
+    # @classmethod
+    # def modify_param_args(cls, params, args, kwargs):
+    #     return params, args, kwargs
+    modify_param_args = None
 
     # method that can be implemented to update parameters via get_param_values before instantiation
     # example:
     # @classmethod
-    # def modify_param_values(cls, params):
-    #     return params
+    # def modify_param_values(cls, values):
+    #     values["some_name"] = "some_value"
+    #     return values
     modify_param_values = None
 
     @classmethod
@@ -142,7 +157,8 @@ class BaseTask(luigi.Task):
         return cls(**cls.req_params(*args, **kwargs))
 
     @classmethod
-    def req_params(cls, inst, _exclude=None, _prefer_cli=None, **kwargs):
+    def req_params(cls, inst, _exclude=None, _prefer_cli=None, _skip_task_excludes=False,
+            _skip_task_excludes_get=None, _skip_task_excludes_set=None, **kwargs):
         # common/intersection params
         params = common_task_params(inst, cls)
 
@@ -151,8 +167,16 @@ class BaseTask(luigi.Task):
 
         # also use this class' req and req_get sets
         # and the req and req_set sets of the instance's class
-        _exclude.update(cls.exclude_params_req, cls.exclude_params_req_get)
-        _exclude.update(inst.exclude_params_req, inst.exclude_params_req_set)
+        # unless explicitly skipped
+        if _skip_task_excludes_get is None:
+            _skip_task_excludes_get = _skip_task_excludes
+        if not _skip_task_excludes_get:
+            _exclude.update(cls.exclude_params_req, cls.exclude_params_req_get)
+
+        if _skip_task_excludes_set is None:
+            _skip_task_excludes_set = _skip_task_excludes
+        if not _skip_task_excludes_set:
+            _exclude.update(inst.exclude_params_req, inst.exclude_params_req_set)
 
         # remove excluded parameters
         for name in list(params.keys()):
@@ -176,15 +200,11 @@ class BaseTask(luigi.Task):
 
         return params
 
-    @classmethod
-    def get_logger_name(cls):
-        return "{}.{}".format(cls.__module__, cls.__name__)
+    def __init__(self, *args, **kwargs):
+        super(BaseTask, self).__init__(*args, **kwargs)
 
-    @classproperty
-    def logger(cls):
-        name = cls.get_logger_name()
-        is_configured = name in logging.root.manager.loggerDict
-        return logging.getLogger(name) if is_configured else setup_logger(name)
+        # task level logger, created lazily
+        self._task_logger = None
 
     def complete(self):
         outputs = [t for t in flatten(self.output()) if not t.optional]
@@ -196,6 +216,25 @@ class BaseTask(luigi.Task):
 
         return all(t.exists() for t in outputs)
 
+    @abstractmethod
+    def run(self):
+        return
+
+    def get_logger_name(self):
+        return self.task_id
+
+    def _create_logger(self, name, level=None):
+        return setup_logger(name, level=level)
+
+    @property
+    def logger(self):
+        if not self._task_logger:
+            name = self.get_logger_name()
+            existing = name in logging.root.manager.loggerDict
+            self._task_logger = logging.getLogger(name) if existing else self._create_logger(name)
+
+        return self._task_logger
+
     @property
     def live_task_id(self):
         """
@@ -253,10 +292,6 @@ class BaseTask(luigi.Task):
 
         return args
 
-    @abstractmethod
-    def run(self):
-        return
-
 
 class Register(BaseRegister):
 
@@ -272,18 +307,22 @@ class Register(BaseRegister):
                     logger.debug("evaluating interactive parameter '{}' with value {}".format(
                         param, value))
                     skip_abort = getattr(inst, "_" + param)(value)
+
+                    # reset the interactive parameter
+                    setattr(inst, param, ())
+
                 except KeyboardInterrupt:
                     print("\naborted")
 
                 # abort the process if not explicitly skipped
                 if not skip_abort:
                     abort(exitcode=0)
+                print("")
 
         return inst
 
 
-@six.add_metaclass(Register)
-class Task(BaseTask):
+class Task(six.with_metaclass(Register, BaseTask)):
 
     log_file = luigi.Parameter(default=NO_STR, significant=False, description="a custom log file; "
         "default: <task.default_log_file>")
@@ -296,26 +335,31 @@ class Task(BaseTask):
         "collections (default: 0), 3. a flag that is passed to the status text creation (default: "
         "'')")
     print_output = CSVParameter(default=(), significant=False, description="print a flat list of "
-        "output targets but do not run any task; this CSV parameter accepts a single integer value "
-        "which sets the task recursion depth (0 means non-recursive)")
+        "output targets but do not run any task; this CSV parameter accepts up to two values: 1. "
+        "the task recursion depth (0 means non-recursive), 2. a boolean flag that decides whether "
+        "paths of file targets should contain file system schemes (default: True)")
     remove_output = CSVParameter(default=(), significant=False, description="remove task outputs "
-        "but do not run any task; this CSV parameter accepts up to three values: 1. the task "
-        "recursion depth (0 means non-recursive), 2. one of the modes 'i' (interactive), 'a' "
-        "(all), 'd' (dry run) (default: 'i'), 3. a flag that decides whether outputs of external "
-        "tasks should be removed (default: False)")
+        "but do not run any task by default; this CSV parameter accepts up to three values: 1. the "
+        "task recursion depth (0 means non-recursive), 2. one of the modes 'i' (interactive), 'a' "
+        "(all), 'd' (dry run) (default: 'i'), 3. a boolean flag that decides whether the task is "
+        "run after outputs were removed (default: False)")
     fetch_output = CSVParameter(default=(), significant=False, description="copy all task outputs "
         "into a local directory but do not run any task; this CSV parameter accepts up to four "
         "values: 1. the task recursion depth (0 means non-recursive), 2. one of the modes 'i' "
         "(interactive), 'a' (all), 'd' (dry run) (default: 'i'), 3. the target directory (default: "
-        "'.'), 4. a flag that decides whether outputs of external tasks should be fetched "
-        "(default: False)")
+        "'.'), 4. a boolean flag that decides whether external outputs and outputs of external "
+        "tasks should be fetched (default: False)")
 
     interactive_params = [
-        "print_deps", "print_status", "print_output", "remove_output", "fetch_output",
+        "print_deps", "print_status", "print_output", "fetch_output", "remove_output",
     ]
 
+    # cache size for published messages
     message_cache_size = 10
 
+    # force skipping this task when remove_output is set to "all" mode
+    skip_output_removal = False
+
     exclude_index = True
     exclude_params_req = set()
     exclude_params_repr = set()
@@ -346,38 +390,49 @@ class Task(BaseTask):
     def is_root_task(self):
         return root_task() == self
 
-    def publish_message(self, msg, scheduler=True):
+    def publish_message(self, msg, stdout=sys.stdout, scheduler=True, **kwargs):
         msg = str(msg)
 
-        sys.stdout.write(msg + "\n")
-        sys.stdout.flush()
+        # write to stdout
+        if stdout:
+            stdout.write(msg + "\n")
+            stdout.flush()
 
+        # publish to the scheduler
         if scheduler:
-            self._publish_message(msg)
+            self._publish_message(msg, **kwargs)
 
-    def _publish_message(self, msg):
-        msg = str(msg)
+    def _publish_message(self, msg, flush_cache=False, silent=False):
+        msg = uncolored(str(msg))
+
+        # flush the message cache?
+        if flush_cache:
+            del self._message_cache[:]
 
         # add to message cache and handle overflow
-        msg = uncolored(msg)
         self._message_cache.append(msg)
         if self.message_cache_size >= 0:
             end = max(len(self._message_cache) - self.message_cache_size, 0)
             del self._message_cache[:end]
 
-        # set status message using the current message cache
+        # set status message based on the full, current message cache
         if callable(getattr(self, "set_status_message", None)):
             self.set_status_message("\n".join(self._message_cache))
-        else:
+        elif not silent:
             logger.warning("set_status_message not set, cannot send task message to scheduler")
 
-    def create_message_stream(self, *args, **kwargs):
+    def _create_message_stream(self, *args, **kwargs):
         return TaskMessageStream(self, *args, **kwargs)
 
+    def _create_logger(self, name, level=None, **kwargs):
+        return setup_logger(name, level=level, add_console_handler={
+            "handler_kwargs": {"stream": self._create_message_stream(**kwargs)},
+        })
+
     @contextmanager
     def publish_step(self, msg, success_message="done", fail_message="failed", runtime=True,
-            scheduler=True):
-        self.publish_message(msg, scheduler=scheduler)
+            scheduler=True, flush_cache=False):
+        self.publish_message(msg, scheduler=scheduler, flush_cache=flush_cache)
         success = False
         t0 = time.time()
         try:
@@ -388,7 +443,7 @@ class Task(BaseTask):
             if runtime:
                 diff = time.time() - t0
                 msg = "{} (took {})".format(msg, human_duration(seconds=diff))
-            self.publish_message(msg, scheduler=scheduler)
+            self.publish_message(msg, scheduler=scheduler, flush_cache=flush_cache)
 
     def publish_progress(self, percentage, precision=1):
         percentage = int(round_discrete(percentage, precision, "floor"))
@@ -423,12 +478,16 @@ class Task(BaseTask):
         return super(Task, self).cli_args(exclude=exclude, replace=replace)
 
     def __repr__(self):
-        return self.repr(color=False)
+        color = Config.instance().get_expanded_boolean("task", "colored_repr")
+        return self.repr(color=color)
+
+    def __str__(self):
+        color = Config.instance().get_expanded_boolean("task", "colored_str")
+        return self.repr(color=color)
 
     def repr(self, all_params=False, color=None, **kwargs):
         if color is None:
-            cfg = Config.instance()
-            color = cfg.get_expanded_boolean("task", "colored_repr")
+            color = Config.instance().get_expanded_boolean("task", "colored_repr")
 
         family = self._repr_family(self.get_task_family(), color=color, **kwargs)
 
@@ -442,19 +501,11 @@ class Task(BaseTask):
 
         return "{}({})".format(family, ", ".join(parts))
 
-    def colored_repr(self, all_params=False):
-        # deprecation warning until v0.1
-        logger.warning("the use of {0}.colored_repr() is deprecated, please use "
-            "{0}.repr(color=True) instead".format(self.__class__.__name__))
-
-        return self.repr(all_params=all_params, color=True)
-
     def _repr_params(self, all_params=False):
         # determine parameters to exclude
         exclude = set()
         if not all_params:
             exclude |= self.exclude_params_repr
-            exclude |= self.inst_exclude_params_repr()
             exclude |= set(self.interactive_params)
 
         # build a map "name -> value" for all significant parameters
@@ -468,25 +519,19 @@ class Task(BaseTask):
     def _repr_flags(self):
         return []
 
-    def inst_exclude_params_repr(self):
-        return set()
-
-    @classmethod
-    def _repr_family(cls, family, color=False, **kwargs):
+    def _repr_family(self, family, color=False, **kwargs):
         return colored(family, "green") if color else family
 
-    @classmethod
-    def _repr_param(cls, name, value, color=False, serialize=True, **kwargs):
+    def _repr_param(self, name, value, color=False, serialize=True, **kwargs):
         # try to serialize first unless explicitly disabled
         if serialize:
-            param = getattr(cls, name, no_value)
+            param = getattr(self.__class__, name, no_value)
             if param != no_value:
                 value = param.serialize(value)
 
         return "{}={}".format(colored(name, color="blue", style="bright") if color else name, value)
 
-    @classmethod
-    def _repr_flag(cls, name, color=False, **kwargs):
+    def _repr_flag(self, name, color=False, **kwargs):
         return colored(name, color="magenta") if color else name
 
     def _print_deps(self, args):
@@ -504,6 +549,37 @@ class Task(BaseTask):
     def _fetch_output(self, args):
         return fetch_task_output(self, *args)
 
+    @classmethod
+    def _law_run_inst(cls, inst, _exclude=None, _replace=None, _global=None, _run_kwargs=None):
+        # get the cli arguments
+        args = inst.cli_args(exclude=_exclude, replace=_replace)
+        args = sum((make_list(tpl) for tpl in args.items()), [])
+
+        # add global parameters when given
+        if _global:
+            args.extend([str(arg) for arg in make_list(_global)])
+
+        # build the full command
+        cmd = [cls.get_task_family()] + args
+
+        # run it
+        return law_run(cmd, **(_run_kwargs or {}))
+
+    @classmethod
+    def law_run_inst(cls, _exclude=None, _replace=None, _global=None, _run_kwargs=None, **kwargs):
+        # create a new instance
+        inst = cls(**kwargs)
+
+        return cls._law_run_inst(inst, _exclude=_exclude, _replace=_replace, _global=_global,
+            _run_kwargs=_run_kwargs)
+
+    def law_run(self, _exclude=None, _replace=None, _global=None, _run_kwargs=None, **kwargs):
+        # when kwargs are given, create a new instance
+        inst = self.req(self, **kwargs) if kwargs else self
+
+        return self._law_run_inst(inst, _exclude=_exclude, _replace=_replace, _global=_global,
+            _run_kwargs=_run_kwargs)
+
     def localize_input(self, *args, **kwargs):
         return localize_file_targets(self.input(), *args, **kwargs)
 
@@ -541,17 +617,18 @@ class ExternalTask(Task):
 
 class TaskMessageStream(BaseStream):
 
-    def __init__(self, task, stdout=True, scheduler=True):
-        super(TaskMessageStream, self).__init__()
+    def __init__(self, task, stdout=sys.stdout, scheduler=True, flush_cache=False, **kwargs):
+        super(TaskMessageStream, self).__init__(**kwargs)
+
         self.task = task
         self.stdout = stdout
         self.scheduler = scheduler
+        self.flush_cache = flush_cache
 
     def _write(self, msg):
-        if self.stdout:
-            self.task.publish_message(msg, scheduler=self.scheduler)
-        elif self.scheduler:
-            self.task._publish_message(msg)
+        # foward to publish_message
+        self.task.publish_message(msg.rstrip("\n"), stdout=self.stdout, scheduler=self.scheduler,
+            flush_cache=self.flush_cache, silent=True)
 
 
 # trailing imports
