@@ -4,18 +4,18 @@
 Helpful utility functions.
 """
 
-
 __all__ = [
     "default_lock", "io_lock", "console_lock", "no_value", "rel_path", "law_src_path",
     "law_home_path", "law_run", "print_err", "abort", "is_number", "try_int", "round_discrete",
     "str_to_int", "flag_to_bool", "common_task_params", "colored", "uncolored", "query_choice",
     "is_pattern", "brace_expand", "range_expand", "range_join", "multi_match", "is_iterable",
-    "is_lazy_iterable", "make_list", "make_tuple", "make_unique", "flatten", "merge_dicts", "which",
-    "map_verbose", "map_struct", "mask_struct", "tmp_file", "interruptable_popen", "readable_popen",
-    "create_hash", "copy_no_perm", "makedirs_perm", "user_owns_file", "iter_chunks", "human_bytes",
-    "parse_bytes", "human_duration", "human_time_diff", "parse_duration", "is_file_exists_error",
-    "send_mail", "DotDict", "ShorthandDict", "open_compat", "patch_object", "join_generators",
-    "quote_cmd", "classproperty", "BaseStream", "TeeStream", "FilteredStream",
+    "is_lazy_iterable", "make_list", "make_tuple", "make_unique", "is_nested", "flatten",
+    "merge_dicts", "which", "map_verbose", "map_struct", "mask_struct", "tmp_file",
+    "interruptable_popen", "readable_popen", "create_hash", "create_random_string", "copy_no_perm",
+    "makedirs", "user_owns_file", "iter_chunks", "human_bytes", "parse_bytes", "human_duration",
+    "human_time_diff", "parse_duration", "is_file_exists_error", "send_mail", "DotDict",
+    "ShorthandDict", "open_compat", "patch_object", "join_generators", "quote_cmd",
+    "escape_markdown", "classproperty", "BaseStream", "TeeStream", "FilteredStream",
 ]
 
 
@@ -30,6 +30,7 @@ import tempfile
 import subprocess
 import signal
 import hashlib
+import uuid
 import shutil
 import copy
 import collections
@@ -109,11 +110,11 @@ def law_home_path(*paths):
     return law_home_path(*paths)
 
 
-def law_run(argv):
+def law_run(argv, **kwargs):
     """
     Runs a task with certain parameters as defined in *argv*, which can be a string or a list of
     strings. It must start with the family of the task to run, followed by the desired parameters.
-    Example:
+    All *kwargs* are forwarded to :py:func:`luigi.interface.run`. Example:
 
     .. code-block:: python
 
@@ -128,10 +129,10 @@ def law_run(argv):
     else:
         argv = [str(arg) for arg in argv]
 
-    # luigis pid locking must be disabled
+    # luigi's pid locking must be disabled
     argv.append("--no-lock")
 
-    return luigi_run(argv)
+    return luigi_run(argv, **kwargs)
 
 
 def print_err(*args, **kwargs):
@@ -223,19 +224,20 @@ def str_to_int(s):
 def flag_to_bool(s, silent=False):
     """
     Takes a string flag *s* and returns whether it evaluates to *True* (values ``"1"``, ``"true"``
-    ``"yes"``, ``"on"``, case-insensitive) or *False* (values ``"0"``, ``"false"``, `"no"``,
-    ``"off"``, case-insensitive). When *s* is already a boolean, it is returned unchanged. An error
-    is thrown when *s* is neither of the allowed values and *silent* is *False*. Otherwise, *None*
-    is returned.
+    ``"yes"``, ``"y"``, ``"on"``, case-insensitive) or *False* (values ``"0"``, ``"false"``,
+    `"no"``, ``"n"``, ``"off"``, case-insensitive). When *s* is already a boolean, it is returned
+    unchanged. An error is thrown when *s* is neither of the allowed values and *silent* is *False*.
+    Otherwise, *None* is returned.
     """
     if isinstance(s, bool):
         return s
     elif isinstance(s, six.string_types):
-        if s.lower() in ("true", "1", "yes", "on"):
+        if s.lower() in ("true", "1", "yes", "y", "on"):
             return True
-        elif s.lower() in ("false", "0", "no", "off"):
+        elif s.lower() in ("false", "0", "no", "n", "off"):
             return False
-    elif silent:
+
+    if silent:
         return None
     else:
         raise ValueError("cannot convert to bool: {}".format(s))
@@ -402,11 +404,12 @@ def is_pattern(s):
     return "*" in s or "?" in s
 
 
-def brace_expand(s, split_csv=False):
+def brace_expand(s, split_csv=False, escape_csv_sep=True):
     """
     Expands brace statements in a string *s* and returns a list containing all possible string
-    combinations. When *split_csv* is *True*, the input string is split by all ``","`` characters
-    located outside braces and the expansion is performed sequentially on all elements. Example:
+    combinations. When *split_csv* is *True*, the input string is split by all comma characters
+    located outside braces, except for escaped ones when *escape_csv_sep* is *True*, and the
+    expansion is performed sequentially on all elements. Example:
 
     .. code-block:: python
 
@@ -438,14 +441,22 @@ def brace_expand(s, split_csv=False):
 
     # take into account csv splitting
     if split_csv:
-        # replace commas in brace statements to avoid splitting
-        br_comma = "__law_brace_comma__"
-        _s = cre.sub(lambda m: m.group(0).replace(",", br_comma), s)
-        # split by real csv commas and start recursion when a comma was found, otherwise continue
+        # replace csv separators in brace statements to avoid splitting
+        br_sep = "__law_brace_csv_sep__"
+        _s = cre.sub(lambda m: m.group(0).replace(",", br_sep), s)
+        # replace escaped commas
+        if escape_csv_sep:
+            escaped_sep = "__law_escaped_csv_sep__"
+            _s = _s.replace(r"\,", escaped_sep)
+        # split by real csv separators except escaped ones when requested,
         parts = _s.split(",")
+        # add back normal commas
+        if escape_csv_sep:
+            parts = [part.replace(escaped_sep, ",") for part in parts]
+        # start recursion when a comma was found, otherwise continue
         if len(parts) > 1:
-            # replace commas in braces again and recurse
-            parts = [part.replace(br_comma, ",") for part in parts]
+            # replace csv separators in braces again and recurse
+            parts = [part.replace(br_sep, ",") for part in parts]
             return sum((brace_expand(part, split_csv=False) for part in parts), [])
 
     # split the string into n sequences with values to expand and n+1 fixed entities
@@ -474,20 +485,27 @@ def brace_expand(s, split_csv=False):
     return res
 
 
-def range_expand(s, min_value=None, max_value=None):
+def range_expand(s, include_end=True, min_value=None, max_value=None, sep=":"):
     """
-    Takes a string or sequence of strings denoting single positive numbers or ranges of positive
-    numbers such as ``"1-3"`` and returns a list of all selected values, inclusive at both edges.
-    One sided range expressions such as ``"-4"`` or ``"4-"`` are also expanded but they require
-    *min_value* and *max_value* to be set (an exception is raised otherwise). Also, when a
-    *min_value* (*max_value*) is given, no value in the returned list of numbers can be smaller
-    (larger). Example:
+    Takes a string, or a sequence of strings in the format ``"1:3"``, or a tuple or a sequence of
+    tuples containing start and stop values of a range and returns a list of all intermediate
+    values. When *include_end* is *True*, the end value is included. One sided range expressions
+    such as ``":4"`` or ``"4:"`` for strings and ``(None, 4)`` or ``(4, None)`` for tuples are also
+    expanded but they require *min_value* and *max_value* to be set (an exception is raised
+    otherwise). Also, when a *min_value* (*max_value*) is given, no value in the returned list of
+    numbers can be smaller (larger). Example:
 
     .. code-block:: python
 
-        range_expand("5-8")
+        range_expand("5:8")
         # -> [5, 6, 7, 8]
 
+        range_expand((6, 9))
+        # -> [6, 7, 8, 9]
+
+        range_expand("5:8", include_end=False)
+        # -> [5, 6, 7]
+
         range_expand(["5-8", "10"])
         # -> [5, 6, 7, 8, 10]
 
@@ -505,16 +523,42 @@ def range_expand(s, min_value=None, max_value=None):
 
     numbers = []
     for s in make_list(s):
-        s = str(s)
-        if "-" in s:
-            # split into start and stop value and handle empty values
-            start, stop = s.split("-", 1)
-            if not start:
+        start, stop, value = None, None, None
+        single_value = False
+
+        if isinstance(s, (tuple, list)):
+            # parse tuple
+            if len(s) == 1:
+                value = s[0]
+                single_value = True
+            elif len(s) == 2:
+                start, stop = s
+            else:
+                raise ValueError("invalid range tuple length: {}".format(s))
+
+        else:
+            # parse as string
+            s = str(s)
+            if sep in s:
+                parts = s.split(sep, 1)
+                start = parts[0] or None
+                stop = parts[1] or None
+            else:
+                value = s
+                single_value = True
+
+        if single_value:
+            # add a single value
+            numbers.append(to_int(value))
+
+        else:
+            # build the range
+            if start is None:
                 if min_value is None:
                     raise Exception("range '{}' with missing start value requires min_value to be "
                         "set".format(s))
                 start = min_value
-            if not stop:
+            if stop is None:
                 if max_value is None:
                     raise Exception("range '{}' with missing stop value requires max_value to be "
                         "set".format(s))
@@ -527,10 +571,7 @@ def range_expand(s, min_value=None, max_value=None):
                 start, stop = stop, start
 
             # add numbers
-            numbers.extend(range(start, stop + 1))
-        else:
-            # no "-" in s, assume it to be a number
-            numbers.append(to_int(s))
+            numbers.extend(range(start, stop + int(bool(include_end))))
 
     # apply min and max cuts when given
     if min_value is not None:
@@ -544,11 +585,13 @@ def range_expand(s, min_value=None, max_value=None):
     return numbers
 
 
-def range_join(numbers, to_str=False):
+def range_join(numbers, to_str=False, sep=",", range_sep=":"):
     """
-    Takes a sequence of positive integer numbers and returns a sequence 1- and 2-tuples, denoting
-    either single numbers or inclusive start and stop values of possible ranges. When *to_str* is
-    *True*, a string is returned in a format consistent to :py:func:`range_expand`. Example:
+    Takes a sequence of positive integer numbers given either as integer or string types, and
+    returns a sequence 1- and 2-tuples, denoting either single numbers or inclusive start and stop
+    values of possible ranges. When *to_str* is *True*, a string is returned in a format consistent
+    to :py:func:`range_expand` with ranges constructed by *range_sep* and merged with *sep*.
+    Example:
 
     .. code-block:: python
 
@@ -559,16 +602,24 @@ def range_join(numbers, to_str=False):
         # -> [(1, 3), (5,), (7, 9)]
 
         range_join([1, 2, 3, 5, 7, 8, 9], to_str=True)
-        # -> "1-3,5,7-9"
+        # -> "1:3,5,7:9"
     """
     if not numbers:
         return "" if to_str else []
 
-    # type check and sort
+    # check type, convert, make unique and sort
+    _numbers = []
     for n in numbers:
-        if not isinstance(n, six.integer_types):
+        if isinstance(n, six.string_types):
+            try:
+                n = int(n)
+            except ValueError:
+                raise ValueError("invalid number format '{}'".format(n))
+        if isinstance(n, six.integer_types):
+            _numbers.append(n)
+        else:
             raise TypeError("cannot handle non-integer value '{}' in numbers to join".format(n))
-    numbers = sorted(numbers)
+    numbers = sorted(set(_numbers))
 
     # iterate through numbers, keep track of last starts and stops and fill a list of range tuples
     ranges = []
@@ -583,7 +634,10 @@ def range_join(numbers, to_str=False):
 
     # convert to string representation
     if to_str:
-        ranges = ",".join(("{}" if len(r) == 1 else "{}-{}").format(*r) for r in ranges)
+        ranges = sep.join(
+            (str(r[0]) if len(r) == 1 else "{1}{0}{2}".format(range_sep, *r))
+            for r in ranges
+        )
 
     return ranges
 
@@ -612,14 +666,20 @@ def is_iterable(obj):
     return True
 
 
+lazy_iter_types = (
+    types.GeneratorType,
+    six.moves.collections_abc.MappingView,
+    six.moves.range,
+    six.moves.map,
+    enumerate,
+)
+
+
 def is_lazy_iterable(obj):
     """
     Returns whether *obj* is iterable lazily, such as generators, range objects, maps, etc.
     """
-    iter_types = (
-        types.GeneratorType, collections.MappingView, six.moves.range, six.moves.map, enumerate,
-    )
-    return isinstance(obj, iter_types)
+    return isinstance(obj, lazy_iter_types)
 
 
 def make_list(obj, cast=True):
@@ -660,7 +720,7 @@ def make_unique(obj):
     raised.
     """
     if not isinstance(obj, (list, tuple)):
-        if is_iterable(obj):
+        if is_iterable(obj) or is_lazy_iterable(obj):
             obj = list(obj)
         else:
             raise TypeError("object is neither list, tuple, nor generic iterable")
@@ -670,6 +730,13 @@ def make_unique(obj):
     return obj.__class__(ret) if isinstance(obj, tuple) else ret
 
 
+def is_nested(obj):
+    """
+    Takes a list or tuple *obj* and checks whether it only contains items of types list and tuple.
+    """
+    return isinstance(obj, (list, tuple)) and all(isinstance(item, (list, tuple)) for item in obj)
+
+
 def flatten(*structs, **kwargs):
     """ flatten(*structs, flatten_dict=True, flatten_list=True, flatten_tuple=True, flatten_set=True)
     Takes one or multiple complex structured objects *structs*, flattens them, and returns a single
@@ -703,11 +770,26 @@ def flatten(*structs, **kwargs):
 
 
 def merge_dicts(*dicts, **kwargs):
-    """ merge_dicts(*dicts, cls=None)
+    """ merge_dicts(*dicts, cls=None, deep=False)
     Takes multiple *dicts* and returns a single merged dict. The merging takes place in order of the
     passed dicts and therefore, values of rear objects have precedence in case of field collisions.
     The class of the returned merged dict is configurable via *cls*. If it is *None*, the class is
     inferred from the first dict object in *dicts*.
+
+    When *deep* is *True*, dictionary types within the dictionaries to merge are updated recursively
+    such that their fields are merged. This is only possible when input dictionaries have a similar
+    structure. Example:
+
+    .. code-block:: python
+
+        merge_dicts({"foo": 1, "bar": {"a": 1, "b": 2}}, {"bar": {"c": 3}})
+        # -> {"foo": 1, "bar": {"c": 3}}  # fully replaced "bar"
+
+        merge_dicts({"foo": 1, "bar": {"a": 1, "b": 2}}, {"bar": {"c": 3}}, deep=True)
+        # -> {"foo": 1, "bar": {"a": 1, "b": 2, "c": 3}}  # inserted entry bar.c
+
+        merge_dicts({"foo": 1, "bar": {"a": 1, "b": 2}}, {"bar": 2}, deep=True)
+        # -> {"foo": 1, "bar": 2}  # "bar" has a different type, so this just uses the rear value
     """
     # get or infer the class
     cls = kwargs.get("cls", None)
@@ -720,9 +802,22 @@ def merge_dicts(*dicts, **kwargs):
             raise TypeError("cannot infer cls as none of the passed objects is of type dict")
 
     # start merging
+    deep = kwargs.get("deep", False)
     merged_dict = cls()
     for d in dicts:
-        if isinstance(d, dict):
+        if not isinstance(d, dict):
+            continue
+
+        if deep:
+            for k, v in d.items():
+                # just take the value as is when it is not a dict, or the field is either not
+                # existing yet or not a dict in the merged dict
+                if not isinstance(v, dict) or not isinstance(merged_dict.get(k), dict):
+                    merged_dict[k] = v
+                else:
+                    # merge by recursion
+                    merged_dict[k] = merge_dicts(merged_dict[k], v, cls=cls, deep=deep)
+        else:
             merged_dict.update(d)
 
     return merged_dict
@@ -899,7 +994,7 @@ def mask_struct(mask, struct, replace=no_value):
         mask_struct({"a": [False, True], "b": False}, struct)
         # => {"a": [2]}
 
-        # omitting mask information results in keeping values
+        # omitting mask information results in kept values
         mask_struct({"a": [False, True]}, struct)
         # => {"a": [2], "b": [3, ["foo", "bar"]]}
     """
@@ -1070,13 +1165,29 @@ def readable_popen(*args, **kwargs):
     return p, line_gen()
 
 
-def create_hash(inp, l=10, algo="sha256"):
+def create_hash(inp, l=10, algo="sha256", to_int=False):
     """
-    Takes an input *inp* and creates a hash based on an algorithm *algo*. For valid algorithms, see
-    python's hashlib. *l* corresponds to the maximum length of the returned hash. Internally, the
-    string representation of *inp* is used.
+    Takes an arbitrary input *inp* and creates a hexadecimal string hash based on an algorithm
+    *algo*. For valid algorithms, see python's hashlib. *l* corresponds to the maximum length of the
+    returned hash and is limited by the length of the hexadecimal representation produced by the
+    hashing algorithm. When *to_int* is *True*, the decimal integer representation is returned.
     """
-    return getattr(hashlib, algo)(six.b(str(inp))).hexdigest()[:l]
+    h = getattr(hashlib, algo)(six.b(str(inp))).hexdigest()[:l]
+    return int(h, 16) if to_int else h
+
+
+def create_random_string(prefix="", l=10):
+    """
+    Creates and returns a random string consisting of *l* characters using a uuid4 hash. When
+    *prefix* is given, the string will have the format ``<prefix>_<random_string>``.
+    """
+    s = ""
+    while len(s) < l:
+        s += uuid.uuid4().hex
+    s = s[:l]
+    if prefix:
+        s = "{}_{}".format(prefix, s)
+    return s
 
 
 def copy_no_perm(src, dst):
@@ -1089,20 +1200,36 @@ def copy_no_perm(src, dst):
     os.chmod(dst, perm)
 
 
-def makedirs_perm(path, perm=None):
+def makedirs(path, perm=None):
     """
-    Recursively creates directory up to *path*. If *perm* is set, the permissions of all newly
-    created directories are set to its value.
+    Recursively creates directories up to *path*. No exception is raised if *path* refers to an
+    existing directory. If *perm* is set, the permissions of all newly created directories are set
+    to this value.
     """
-    if not os.path.exists(path):
-        if perm is None:
-            os.makedirs(path)
-        else:
-            umask = os.umask(0)
-            try:
+    # nothing to do when the directory already exists
+    if os.path.isdir(path):
+        return
+
+    # helper to silently create the directory, catching exceptions if it exists by now
+    # (when dropping py2, just use the exist_ok flag of os.makedirs)
+    def makedirs_safe(path, perm=None):
+        try:
+            if perm is None:
+                os.makedirs(path)
+            else:
                 os.makedirs(path, perm)
-            finally:
-                os.umask(umask)
+        except Exception as e:
+            if not is_file_exists_error(e):
+                raise
+
+    if perm is None:
+        makedirs_safe(path)
+    else:
+        umask = os.umask(0)
+        try:
+            makedirs_safe(path, perm)
+        finally:
+            os.umask(umask)
 
 
 def user_owns_file(path, uid=None):
@@ -1766,6 +1893,14 @@ def quote_cmd(cmd):
     return " ".join(six.moves.shlex_quote(part) for part in cmd)
 
 
+def escape_markdown(s):
+    """
+    Escapes all characters in a string *s* that coupld be confused for markdown formatting strings
+    and returns it.
+    """
+    return re.sub(r"([^\\]?)(\(|\)|=|\.|_|-)", r"\1\\\2", s)
+
+
 class ClassPropertyDescriptor(object):
     """
     Generic descriptor class that is used by :py:func:`classproperty`. Setters are currently not
@@ -1805,9 +1940,15 @@ class BaseStream(object):
 
     FLUSH_AFTER_WRITE = True
 
-    def __init__(self):
+    def __init__(self, flush_after_write=None):
         super(BaseStream, self).__init__()
+
         self.closed = False
+        self.flush_after_write = flush_after_write
+
+    @property
+    def _flush_after_write(self):
+        return self.FLUSH_AFTER_WRITE if self.flush_after_write is None else self.flush_after_write
 
     def __del__(self):
         self.close()
@@ -1821,6 +1962,7 @@ class BaseStream(object):
     def close(self):
         if not self.closed:
             self.flush()
+
             self._close()
             self.closed = True
 
@@ -1831,7 +1973,8 @@ class BaseStream(object):
     def write(self, *args, **kwargs):
         if not self.closed:
             self._write(*args, **kwargs)
-            if self.FLUSH_AFTER_WRITE:
+
+            if self._flush_after_write:
                 self.flush()
 
     def _close(self):
@@ -1845,10 +1988,13 @@ class BaseStream(object):
 
 
 class TeeStream(BaseStream):
-    """ __init__(*consumers, mode="w")
+    """ __init__(*consumers, mode="w", **kwargs)
     Multi-stream object that forwards calls to :py:meth:`write` and :py:meth:`flush` to all
     registered *consumer* streams. When a *consumer* is a string, it is interpreted as a file which
-    is opened for writing (similar to *tee* in bash). Example:
+    is opened for writing (similar to *tee* in bash). All *kwargs* are forwarded to the
+    :py:class:`BaseStream` constructor.
+
+    Example:
 
     .. code-block:: python
 
@@ -1857,9 +2003,9 @@ class TeeStream(BaseStream):
     """
 
     def __init__(self, *consumers, **kwargs):
-        super(TeeStream, self).__init__()
+        mode = kwargs.pop("mode", "w")
 
-        mode = kwargs.get("mode", "w")
+        super(TeeStream, self).__init__(**kwargs)
 
         self.consumers = []
         self.open_files = []
@@ -1898,11 +2044,11 @@ class FilteredStream(BaseStream):
     """
     Stream object that accepts in input *stream* and a function *filter_fn* which is called upon
     every call to :py:meth:`write`. The payload is written when the returned value evaluates to
-    *True*.
+    *True*. All *kwargs* are forwarded to the :py:class:`BaseStream` constructor.
     """
 
-    def __init__(self, stream, filter_fn):
-        super(FilteredStream, self).__init__()
+    def __init__(self, stream, filter_fn, **kwargs):
+        super(FilteredStream, self).__init__(**kwargs)
         self.stream = stream
         self.filter_fn = filter_fn
 
